#ifndef _RGB_HPP_
#define _RGB_HPP_

#include <iostream>

namespace Color {

  // Floating RGB color
  struct RGB {
    //-- Methods --
    // CTOR
    RGB();
    RGB(const uint8_t&, const uint8_t&, const uint8_t&);
    RGB(const RGB&);
    // DTOR
    ~RGB();

    // operators
    RGB& operator= (const RGB&);
    RGB& operator+=(const RGB&);
    RGB& operator-=(const RGB&);
    RGB& operator*=(const RGB&);
    RGB& operator*=(const uint8_t&);
    RGB& operator/=(const RGB&);
    RGB& operator/=(const uint8_t&);

    friend RGB operator+ (const RGB&,     const RGB&);
    friend RGB operator- (const RGB&,     const RGB&);
    friend RGB operator* (const RGB&,     const RGB&);
    friend RGB operator* (const RGB&,     const uint8_t&);
    friend RGB operator* (const uint8_t&, const RGB&);
    friend RGB operator/ (const RGB&,     const RGB&);
    friend RGB operator/ (const RGB&,     const uint8_t&);

    friend std::ostream& operator<< (std::ostream&, const RGB&);

    // Euclidian distance between 2 colors
    friend uint32_t euclidian(const RGB&, const RGB&);
    // Manhattan distance between 2 colors
    friend uint32_t manhattan(const RGB&, const RGB&);

    //-- Members --
    union {
      struct {
	uint8_t r, g, b;
      };
      uint8_t c[3];
    };
  };

};

#endif // _RGB_HPP_
